@@ -2,13 +2,13 @@ import os
 import sys
 import json
 import subprocess
-from packaging.version import Version
 from pathlib import Path
 
-import pytest
 import matplotlib
 import matplotlib.ft2font
 import matplotlib.pyplot as plt
+import pytest
+from packaging.version import Version
 
 MPL_VERSION = Version(matplotlib.__version__)
 
@@ -291,7 +291,7 @@ def test_hash_fails(tmpdir):
     # If we use --mpl, it should detect that the figure is wrong
     output = assert_pytest_fails_with(['--mpl', test_file], "doesn't match hash FAIL in library")
     # We didn't specify a baseline dir so we shouldn't attempt to find one
-    assert "Unable to find baseline image" not in output, output
+    assert "Image file not found for comparison test" not in output, output
 
     # Check that the summary path is printed and that it exists.
     output = assert_pytest_fails_with(['--mpl', test_file, '--mpl-generate-summary=html'],
@@ -337,13 +337,13 @@ def test_hash_fail_hybrid(tmpdir):
     output = assert_pytest_fails_with(['--mpl', test_file,
                                        '--mpl-baseline-path=/not/a/path'],
                                       "doesn't match hash FAIL in library")
-    assert "Unable to find baseline image" in output, output
+    assert "Image file not found for comparison test" in output, output
 
     # Assert reports image comparison succeeds
     output = assert_pytest_fails_with(['--mpl', test_file,
                                        rf'--mpl-baseline-path={hash_baseline_dir_abs / "succeed"}'],
                                       "doesn't match hash FAIL in library")
-    assert "However, the comparison to the baseline image succeeded." in output, output
+    assert "The comparison to the baseline image succeeded." in output, output
 
     # If we don't use --mpl option, the test should succeed
     code = call_pytest([test_file])
@@ -411,3 +411,78 @@ def test_hash_missing(tmpdir):
     # If we don't use --mpl option, the test should succeed
     code = call_pytest([test_file])
     assert code == 0
+
+
+TEST_RESULTS_ALWAYS = """
+import pytest
+import matplotlib.pyplot as plt
+def plot():
+    fig = plt.figure()
+    ax = fig.add_subplot(1,1,1)
+    ax.plot([1,2,2])
+    return fig
+@pytest.mark.mpl_image_compare
+def test_modified(): return plot()
+@pytest.mark.mpl_image_compare
+def test_new(): return plot()
+@pytest.mark.mpl_image_compare
+def test_unmodified(): return plot()
+"""
+
+
+@pytest.mark.skipif(not hash_library.exists(), reason="No hash library for this mpl version")
+def test_results_always(tmpdir):
+
+    test_file = tmpdir.join('test.py').strpath
+    with open(test_file, 'w') as f:
+        f.write(TEST_RESULTS_ALWAYS)
+    results_path = tmpdir.mkdir('results')
+
+    code = call_pytest(['--mpl', test_file, '--mpl-results-always',
+                        rf'--mpl-hash-library={hash_library}',
+                        rf'--mpl-baseline-path={baseline_dir_abs}',
+                        '--mpl-generate-summary=html,json,basic-html',
+                        rf'--mpl-results-path={results_path.strpath}'])
+    assert code == 0  # hashes correct, so all should pass
+
+    # assert files for interactive HTML exist
+    assert results_path.join('fig_comparison.html').exists()
+    assert results_path.join('styles.css').exists()
+    assert results_path.join('extra.js').exists()
+
+    comparison_file = results_path.join('fig_comparison_basic.html')
+    with open(comparison_file, 'r') as f:
+        html = f.read()
+
+    json_file = results_path.join('results.json')
+    with open(json_file, 'r') as f:
+        json_results = json.load(f)
+
+    # each test, and which images should exist
+    for test, exists in [
+        ('test_modified', ['baseline', 'result-failed-diff', 'result']),
+        ('test_new', ['result']),
+        ('test_unmodified', ['baseline', 'result']),
+    ]:
+
+        test_name = f'test.{test}'
+
+        summary = f'<div class="test-name">{test_name.split(".")[-1]}</div>'
+        assert summary in html
+
+        assert test_name in json_results.keys()
+        json_res = json_results[test_name]
+        assert json_res['status'] == 'passed'
+
+        for image_type in ['baseline', 'result-failed-diff', 'result']:
+            image = f'{test_name}/{image_type}.png'
+            image_exists = results_path.join(*image.split('/')).exists()
+            json_image_key = f"{image_type.split('-')[-1]}_image"
+            if image_type in exists:  # assert image so pytest prints it on error
+                assert image and image_exists
+                assert image in html
+                assert json_res[json_image_key] == image
+            else:
+                assert image and not image_exists
+                assert image not in html
+                assert json_res[json_image_key] is None
